// This is an incomplete & imprecice implementation of the
// standard file by the same name

#ifndef ENDIAN_H
#define ENDIAN_H

#define __LITTLE_ENDIAN        1234
#define __BIG_ENDIAN           4321

#define __BYTE_ORDER __LITTLE_ENDIAN

#endif // ENDIAN_H
